DataBase

Struct DataBase 

pub struct DataBase { /* private fields */ }

Implementations

Source

impl DataBase

pub fn new(path: &str) -> Self

Creates a new instance of the database or uses the existing db file, at the given path.

Example
let db = mu_db::DataBase::new("./test.db");

Generates (./test.db) and (./index_test.db) if doesn’t exist.

pub fn insert(&mut self, key: &str, value: &str)

Inserts a key-value pair into the database, replacing old value if key exists.

Example
let mut db = mu_db::DataBase::new("./test.db");
db.insert("key", "before");
db.insert("key", "after");
assert_eq!(db.get("key"), Some("after".to_string()));
pub fn get(&mut self, key: &str) -> Option<String>

Retrieves the value associated with the given key from the database.

Example
let mut db = mu_db::DataBase::new("./test.db");
db.insert("key", "value");
assert_eq!(db.get("key"), Some("value".to_string()));
pub fn remove(&mut self, key: &str)

Removes the entry associated with the given key from the index if the key exists. This method does not remove the value in the database file. To completely remove the value, you need to use (.shrink()) after removing the entry.

Example
let mut db = mu_db::DataBase::new("./test.db");
db.insert("key", "value");
assert_eq!(db.get("key"), Some("value".to_string()));
db.remove("key");
assert_eq!(db.get("key"), None);
pub fn clear_all(&mut self) -> Result<()>

Clears all data in the database.

Example
let mut db = mu_db::DataBase::new("./test.db");
db.insert("key", "value");
assert!(!db.is_empty());
assert!(!db.is_buf_empty());
db.clear_all().unwrap();
assert!(db.is_empty());
assert!(db.is_buf_empty());
pub fn shrink(&mut self)

Optimizes the database file by removing any unused space.

Example
let mut db = mu_db::DataBase::new("./test.db");
db.clear_all();
db.insert("k1", "1".repeat(10).as_str());
db.insert("k2", "2".repeat(10).as_str());
assert_eq!(db.buf_len(), 20);
db.remove("k1");
assert_eq!(db.buf_len(), 20);
db.insert("k3", "3".repeat(5).as_str());
assert_eq!(db.buf_len(), 20);
db.shrink();
assert_eq!(db.buf_len(), 15);
db.remove("k2");
db.remove("k3");
assert_eq!(db.buf_len(), 15);
db.shrink();
assert_eq!(db.buf_len(), 0);
pub fn read_at(&mut self, start: u64, size: usize) -> Result<String>

Reads data directly from the database file at the specified position (start) and size (size).

Example
let mut db = mu_db::DataBase::new("./test.db");
db.clear_all();
db.insert("k1", "hello");
db.insert("k2", "world");
assert_eq!(db.read_at(5, 5).unwrap(), "world".to_string());
pub fn write_at(&mut self, start: u64, content: &str) -> Result<()>

Writes data directly to the database file at the specified position with any length.

Example
let mut db = mu_db::DataBase::new("./test.db");
db.clear_all();
db.write_at(5, "world").unwrap();
assert_eq!(db.read_at(5, 5).unwrap(), "world".to_string());
pub fn is_empty(&self) -> bool

Returns true if self.index.entries is empty, and false otherwise.

If you want to know if db file is empty, use (.is_buf_empty()).

Example
let mut db = mu_db::DataBase::new("./test.db");
db.clear_all();
db.insert("key", "value");
assert!(!db.is_empty());
assert!(!db.is_buf_empty());
db.remove("key");
assert!(db.is_empty());
assert!(!db.is_buf_empty());
db.shrink();
assert!(db.is_empty());
assert!(db.is_buf_empty());
pub fn is_buf_empty(&self) -> bool

Returns true if db file has metadata length of 0, and false otherwise.

Example
let mut db = mu_db::DataBase::new("./test.db");
db.clear_all();
assert!(db.is_buf_empty());
db.insert("key", "value");
assert!(!db.is_buf_empty());
pub fn buf_len(&self) -> u64

Returns the length of the db file matadata.

Example
let mut db = mu_db::DataBase::new("./test.db");
db.clear_all();
db.insert("key", "value");
assert_eq!(db.buf_len(), 5);
db.clear_all();
assert_eq!(db.buf_len(), 0);
pub fn set_buf_len(&mut self, len: u64)

Sets the length of the database file directly, truncating or extending it as necessary.

Example
let mut db = mu_db::DataBase::new("./test.db");
db.clear_all();
assert!(db.is_buf_empty());
assert_eq!(db.buf_len(), 0);
db.insert("key", "value");
assert_eq!(db.buf_len(), 5);
assert!(!db.is_buf_empty());
db.set_buf_len(0);
assert_eq!(db.buf_len(), 0);
assert!(db.is_buf_empty());